;------------------------------------------------------------------------------
;
;  LCD-Display ber SPI-Schnittstelle mit CD4094
;
;  CD4094
;  pin11 Q8 = DB7
;  pin12 Q7 = DB6
;  pin13 Q6 = DB5
;  pin14 Q5 = DB4
;  pin7  Q4 = ENABLE
;  pin6  Q3 = frei
;  pin5  Q2 = RS
;  pin4  Q1 = BL
;
;  pin1  Strobe = SS
;  pin2  Dat    = MOSI
;  pin3  CLK    = SCLK
;
;  S.Hofmann
;  rev. 0.1
;
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------ 
; Datenbyte zum LCD senden
;------------------------------------------------------------------------------
SendDat:	

           sbr   data_4094, RS          ; RS = high
           rcall Ausgabe
           ret

;------------------------------------------------------------------------------ 
; Befehl zum LCD senden
;------------------------------------------------------------------------------
SendCom:                            
           cbr   data_4094, RS          ; RS = low
           rcall Ausgabe
           ret
;------------------------------------------------------------------------------ 
; Ausgebe ber SPI
;------------------------------------------------------------------------------
Ausgabe:
           mov   temp2, temp1           ; retten 
           andi  temp1, 0b11110000      ; erst das hherwertige Nippel
           andi  data_4094,0b00001111   ; alte Daten ungltig
           or    data_4094, temp1       ; in data
           rcall out_4094               ; Ausgabe
           rcall lcd_enable             ; Enable high --> low
           swap  temp2                  ; Low Nippel nach oben
           andi  temp2, 0b11110000      ; 
           andi  data_4094, 0b00001111  ; alte Daten ungltig
           or    data_4094, temp2       ; in data
           rcall out_4094               ; Ausgabe
           rcall lcd_enable             ;Enable-Routine aufrufen
           rcall delay50us              ;Delay-Routine aufrufen
           ret                          ;zurck zum Hauptprogramm

;------------------------------------------------------------------------------ 
; erzeugt den Enable-Puls
;------------------------------------------------------------------------------
lcd_enable:
           sbr data_4094,ENABLE          ;Enable high
           rcall out_4094
           nop                           ;
           cbr data_4094,ENABLE          ;Enable wieder low
           rcall out_4094
           ret                           ;Und wieder zurck                                 
;------------------------------------------------------------------------------ 
; Gibt <cnt> Zeichen an das Display aus
;------------------------------------------------------------------------------
OutText:
           lsl   ZL
           rol   ZH	                     ; Z-Ptr um 1 Pos. nach links
OT1:
           lpm	                          ; Zeichen-Code nach r0 laden
           mov   temp1,r0	               ; Zeichen-Code nach prm1 kopieren
           rcall SendDat	                ; Zeichen senden, Autoinkrement der Adresse
           adiw  ZL,1	                   ; Z-Ptr inkrementieren
           dec   cnt	                    ; Zhler - 1
           brne  OT1	                    ; alle Zeichen an das LCD ausgeben
           ret
;------------------------------------------------------------------------------ 
; ; Def. Sonderzeichen des Anwenders, Char.-Code in temp1
;------------------------------------------------------------------------------
DefChar:	
           lsl   temp1	          ; temp1 mit 2 multipliziert
           lsl   temp1	          ; temp1 mit 4 multipliziert
           ldi   ZH,high(CharTab) ; high-Byte der Punktemuster-Tabellenadr.
           lsl   temp1	          ; temp1 mit 8 multipl. (8 Zeilen pro Zeichen CG-RAM)
           mov   ZL,temp1	  ; Z-Ptr zeigt auf Anf.-Adr. des Zeichens in der Char.-Tab 
           rol   ZH	          ; Z-Ptr um 1 Pos. nach links schieben
           sbr   temp1,1<<6	  ; Bit6 setzen -> Zieladr. ist im CG-RAM
           rcall SendCom	  ; Adressenzhler im CG-RAM setzen
           ldi   cnt,8
DC1:
           lpm	                  ; Punktemuster nach r0 laden
           mov   temp1,r0	  ; Punktemuster nach prm1 kopieren
           rcall SendDat	  ; Zeichenmuster senden, Autoinkrement der Adresse
           adiw  ZL,1	          ; Z-Ptr inkrementieren
           dec   cnt	          ; Zhler - 1
           brne  DC1	          ; alle 8 Zeilen ins CG-RAM kopieren
           ret
;------------------------------------------------------------------------------ 
; Pause nach jeder bertragung
;------------------------------------------------------------------------------
delay50us:                              ;50us Pause
           ldi  temp1, $42
delay50us_:dec  temp1
           brne delay50us_
           ret                          ;wieder zurck
;------------------------------------------------------------------------------ 
; Lngere Pause fr manche Befehle
;------------------------------------------------------------------------------
delay5ms:                               ;5ms Pause
           ldi  temp1, $21
WGLOOP0:   ldi  temp2, $C9
WGLOOP1:   dec  temp2
           brne WGLOOP1
           dec  temp1
           brne WGLOOP0
           ret                          ;wieder zurck
;------------------------------------------------------------------------------ 
; Lngere Pause 5s
;------------------------------------------------------------------------------
Wait5s:	; @ Systemtakt = 4 MHz
           ldi   cnt,130	; Zhler laden
Wait5s1:
           rcall delay5ms
           rcall delay5ms
           rcall delay5ms
           rcall delay5ms
           rcall delay5ms
           rcall delay5ms
           rcall delay5ms
           rcall delay5ms
           dec   cnt
           brne  Wait5s1
           ret
;------------------------------------------------------------------------------ 
; Initialisierung: muss ganz am Anfang des Programms aufgerufen werden
;------------------------------------------------------------------------------
lcd_init:
           ldi   data_4094,0x00         ; init schieberegister
           rcall out_4094
           ldi   temp3,50
powerupwait:
           rcall delay5ms
           dec   temp3
           brne	 powerupwait
           ldi   data_4094, 0b00110000  ;muss 3mal hintereinander gesendet 
           rcall out_4094           
           rcall lcd_enable             ;1
           rcall delay5ms
           rcall lcd_enable             ;2
           rcall delay5ms
           rcall lcd_enable             ;und 3!
           rcall delay5ms
           ldi   data_4094, 0b00100000  ;4bit-Modus einstellen  
           rcall out_4094
           rcall lcd_enable
           rcall delay5ms
           ldi   temp1, 0b00101000      ;noch was einstellen...
           rcall SendCom
           ldi   temp1, 0b00001100      ;...nochwas...
           rcall SendCom
           ldi   temp1, 0b00000100      ;endlich fertig
           rcall SendCom
           ret
;------------------------------------------------------------------------------ 
; Sendet den Befehl zur Lschung des Displays
;------------------------------------------------------------------------------
lcd_clear:
           ldi temp1, 0b00000001   ;Display lschen
           rcall SendCom
           rcall delay5ms
           ret
;------------------------------------------------------------------------------ 
; Licht an
;------------------------------------------------------------------------------
backlight_on:
           sbr data_4094,BL          ; BL on
           rcall out_4094
           ret
;------------------------------------------------------------------------------ 
; Licht aus
;------------------------------------------------------------------------------
backlight_off:
           cbr data_4094,BL          ; BL off
           rcall out_4094
           ret
;------------------------------------------------------------------------------ 
; Ausgabe ber SPI
;------------------------------------------------------------------------------
out_4094:
   push  data_4094
   cbi	  LCD_PORT,Strobe        ; Strobe auf Low
   out   SPDR, data_4094
spil:
   sbis  SPSR, SPIF
   rjmp  spil                    ; jump wenn nicht fertig
   sbi	 LCD_PORT,Strobe         ; Strobe High
   nop
   cbi	 LCD_PORT,Strobe         ; Strobe Low Daten OK
   pop   data_4094
   ret	                         ; return   
;------------------------------------------------------------------------------ 
; Schieberegister CD4094 laden ohne SPI-Controller
;------------------------------------------------------------------------------
;out_4094:                       ; entweder SPI oder normal
	  cli
	  push  data_4094   
	  cbi	LCD_PORT,Strobe	          ; Strobe auf Low
	  cbi	LCD_PORT,clk	             ; clk auf low
	  ldi	temp3,8		                 ; 8Bit Schieben
putchar0: 
	  lsl	data_4094
	  brcs	putchar1	                ;If carry set	
	  cbi	LCD_PORT,D4094	           ;    send a '0'	
	  rjmp	putchar2	                ;else	
putchar1:
	  sbi	LCD_PORT,D4094	           ;    send a '1'	
	  nop
putchar2:
	  sbi	LCD_PORT,clk	             ; Clock High
          
   nop
   nop
                  	  
	  cbi	LCD_PORT,clk              ; Clock Low
			                              ; Get next bit	
	  dec	temp3		                   ; If not all bit sent	
	  brne	putchar0	                ;   send next	
				                             ; else
	  sbi	LCD_PORT,Strobe           ; Strobe High
          
   nop
                                                         
	  cbi	LCD_PORT,Strobe           ; Strobe Low Daten OK
	  pop   data_4094
	  sei
	  ret			                        ; return

.org  $400
CharTab:
.db   $04,$04,$1F,$04,$04,$00,$1F,$00 	; Ch.-Code $00: Zeichen "+/-"
.db   $0C,$02,$04,$08,$0E,$00,$00,$00 	; Ch.-Code $01: Zeichen "^2"
.db   $0C,$02,$0C,$02,$0C,$00,$00,$00 	; Ch.-Code $02: Zeichen "^3"
.db   $18,$06,$01,$06,$18,$00,$1F,$00 	; Ch.-Code $03: Zeichen ">="
.db   $03,$0C,$10,$0C,$03,$00,$1F,$00 	; Ch.-Code $04: Zeichen "<="
.db   $04,$04,$1F,$04,$1F,$04,$04,$00 	; Ch.-Code $05: Zeichen "ungleich"
.db   $00,$1F,$00,$1F,$00,$1F,$00,$00 	; Ch.-Code $06: Zeichen "identisch"
.db   $15,$15,$15,$15,$15,$15,$15,$00 	; Ch.-Code $06: Zeichen "3 Balken"
Text1:
.db   "Sonderzeichen:  "
Text2:
.db   0,' ',1,' ',2,' ',3,' ',4,' ',5,' ',6,' ',7,' '
Text3:
.db   "  Display-Test  "
Text4:
.db   "Char 6-8 blinken"
Text5:
.db   "Cursor-Pos. = 65"